#!/usr/bin/python
#
# RASP: RASP Aids Song Production
# Version 2.1
# Copyright 2012, Brian Hilmers
# License: OSL 3.0
# Python 2.6
#
# This is a collection of handy tools for electronic music production:
# Tap Tempo, a Song Time calculator, and a Delay/Hertz calculator for each
# note division. In addition, there is a Metronome and Frequency Generator.
# These are the tools I use most when writing, all in one place!
#
# Depends on Tkinter, tksnack, and maybe libsnack2, or libsnack2-alsa (2.2.10)

from time import time, sleep
from threading import Thread
from Tkinter import *
import tkMessageBox
import tkFont
import tkSnack


# Initialize
root = Tk()
tkSnack.initializeSnack(root)

# Variables for tap tempo
tap_time = time()
previous_time = time()
tap_count = 0

# Variables for the metronome
play = True

# Metronome functions
def metro_start():
    # Change the button text and start a new thread for the metronome
    global play
    if metro_button["text"] == "Start":
        metro_button["text"] = "Stop"
        gen_stop() # Stop the frequency generator
        play = True
        thread = Thread(target = metro_play)
        thread.start()
    else:
        play = False
        metro_button["text"] = "Start"
        tkSnack.audio.stop()

def metro_play():
    validate()
    metro_sound = tkSnack.Sound(rate=44100)
    metro_bpm = float(input_bpm.get())
    metro_time = (60.0 / metro_bpm)
    metro_time = adjust_beat_value(metro_time)
    base_time = time()
    swing_time = base_time + metro_time
    first_click = 1
    first_beat = downbeat.get()
    beat_tracker = 1
    
    while play == True:
        validate()
        try:
            if float(input_bpm.get()) <= 0:
                input_bpm.delete(0, END)
                input_bpm.insert(0, 1)
                metro_bpm = 1
            else:
                metro_bpm = float(input_bpm.get())
        except:
            metro_bpm = 1
            input_bpm.delete(0, END)
            input_bpm.insert(0, 1)
        
        # Set metronome to a maximum of 250 BPM
        if float(input_bpm.get()) > 250:
            metro_bpm = 250
            input_bpm.delete(0, END)
            input_bpm.insert(0, "250")
        else:
            metro_bpm = float(input_bpm.get())
            
        metro_time = (60.0 / metro_bpm)
        metro_time = adjust_beat_value(metro_time)
        metro_vol = metro_vol_slider.get() * 3275
        click1 = tkSnack.Filter('generator', 880, metro_vol, 0.0, 'sine', 400)
        click2 = tkSnack.Filter('generator', 1760, metro_vol, 0.0, 'sine', 400)
        # There is probably no need to destroy these filters after every event
        # but I'm going to anyway. It seems to keep down memory use, slightly.
        # Filter(filter type frequency amplitude(max is 32767) shape type ntot)
        # http://www.speech.kth.se/snack/man/snack2.2/tcl-man.html#filter

        if first_click == 1:
            if first_beat == 1:
                metro_sound.play(filter=click1, blocking=1);click1.destroy()
                first_click = 0
                first_beat = 0
                sleep(metro_time - .01) # .01 is about 400 samples @ 41000...
            else:
                metro_sound.play(filter=click2, blocking=1);click2.destroy()
                first_click = 0
                sleep(metro_time - .01) # ...and seems to help the start up lag

        if base_time > swing_time:
            if first_beat == 1:
                if downbeat.get() == 1:
                    metro_sound.play(filter=click1, blocking=1);click1.destroy()
                    beat_tracker = 1
                    first_beat = 0
                else:
                    metro_sound.play(filter=click2, blocking=1);click2.destroy()
                    beat_tracker += 1
            else:
                metro_sound.play(filter=click2, blocking=1);click2.destroy()
                beat_tracker += 1
                if beat_tracker == int(input_beats_per_measure.get()):
                    beat_tracker = 1
                    first_beat = 1 
            sync_time = base_time - swing_time
            swing_time = base_time + metro_time - sync_time
            sleep(metro_time - sync_time)
        else:
            base_time = time()

def adjust_beat_value(metro_time):
    if int(input_beat_value.get()) == 1: metro_time *= 4
    elif int(input_beat_value.get()) == 2: metro_time *= 2
    elif int(input_beat_value.get()) == 4: metro_time *= 1
    elif int(input_beat_value.get()) == 8: metro_time *= .5
    elif int(input_beat_value.get()) == 16: metro_time *= .25
    elif int(input_beat_value.get()) == 32: metro_time *= .125
    elif int(input_beat_value.get()) == 64: metro_time *= .0625
    return metro_time

def tempo_markings(mark):
    # Taken from Nikko 200 Series metronomes
    if mark == "Grave": new_tempo = 40
    elif mark == "Largo": new_tempo = 44
    elif mark == "Lento": new_tempo = 52
    elif mark == "Adagio": new_tempo = 56
    elif mark == "Larghetto": new_tempo = 60
    elif mark == "Adagietto": new_tempo = 66
    elif mark == "Andante": new_tempo = 72
    elif mark == "Andantino": new_tempo = 80
    elif mark == "Maestoso": new_tempo = 88
    elif mark == "Moderato": new_tempo = 96
    elif mark == "Allegretto": new_tempo = 108
    elif mark == "Animato": new_tempo = 120
    elif mark == "Allegro": new_tempo = 132
    elif mark == "Assai": new_tempo = 144
    elif mark == "Vivace": new_tempo = 160
    elif mark == "Presto": new_tempo = 184
    elif mark == "Prestissimo": new_tempo = 208
    input_bpm.delete(0, END),input_bpm.insert(0, new_tempo)

# BPM functions
def tap(arg):
    # This function is a complete rewrite after 2.1, modeled
    # after Richard Reel's BPM Tool on http://www.all8.com
    global previous_time 
    global tap_time
    global tap_count
    current_time = time()
    bpm = 0
    
    if (current_time - previous_time) > 2:
        tap_count = 0
    if tap_count == 0:
        tap_time = time()
        tap_count = 1
    else:
        bpm = round(60 * tap_count / (current_time - tap_time), 1)
        tap_count += 1
        
    previous_time = current_time
    input_bpm.delete(0, END)
    input_bpm.insert(0, bpm)

def arrow(event):
    if event.keysym == "Up":
        v = float(input_bpm.get())
        v += 1
        input_bpm.delete(0, END)
        input_bpm.insert(0, v)
    elif event.keysym == "Down":
        v = float(input_bpm.get())
        v -= 1
        input_bpm.delete(0, END)
        input_bpm.insert(0, v)
    elif event.keysym == "Left":
        v = float(input_bpm.get())
        v -= 10
        input_bpm.delete(0, END)
        input_bpm.insert(0, v)
    elif event.keysym == "Right":
        v = float(input_bpm.get())
        v += 10
        input_bpm.delete(0, END)
        input_bpm.insert(0, v)

# Delay calculator functions
def calculate():
    # This is a whole bunch of convering input to
    # other numbers and smashing them together. 
    validate()
    # Transpose the beat value
    if int(input_beat_value.get()) == 1: beat_value = 4
    elif int(input_beat_value.get()) == 2: beat_value = 2
    elif int(input_beat_value.get()) == 4: beat_value = 1
    elif int(input_beat_value.get()) == 8: beat_value = .5
    elif int(input_beat_value.get()) == 16: beat_value = .25
    elif int(input_beat_value.get()) == 32: beat_value = .125
    elif int(input_beat_value.get()) == 64: beat_value = .0625
    
    seconds_per_beat = 60 / float(input_bpm.get())
    seconds_per_measure = (seconds_per_beat * 
             float(input_beats_per_measure.get()) * beat_value)
    
    # Song time calculation
    song_time = (seconds_per_measure * float(input_total_measures.get())) / 60
    # Split into minutes and seconds
    song_time_minutes = int(song_time)
    song_time_remainder = song_time - song_time_minutes
    song_time_seconds = int(round(60 * song_time_remainder))
    
    # Print out to song time label
    print_song_time = "%i:%02i" % (song_time_minutes, song_time_seconds)
    song_time_output.set(print_song_time)
    
    # Print out to the delay/hertz labels
    if delay_range.get() == 0:
        dcalc_1.set(populate(.0625, "16/1"))
        dcalc_2.set(populate(.125, "8/1"))
        dcalc_3.set(populate(.25, "4/1"))
        dcalc_4.set(populate(.3333, "3/1"))
        dcalc_5.set(populate(.5, "2/1"))
        dcalc_6.set(populate(1, "1/1"))
    else:    
        dcalc_1.set(populate(2, "1/2"))
        dcalc_2.set(populate(4, "1/4"))
        dcalc_3.set(populate(8, "1/8"))
        dcalc_4.set(populate(16, "1/16"))
        dcalc_5.set(populate(32, "1/32"))
        dcalc_6.set(populate(64, "1/64"))

def populate(divisor, divisor_string):
    # The delay and hertz calculation. Get a few basic values, then put the
    # modified versions right into a nicely formated string. Combine all the
    # strings for easy output to the tkLabel. Note: "divisor" is really
    # "input_beat_value" from above, minus whole notes.
    milliseconds = (((60 * 1000) / float(input_bpm.get())) * 
                   (float(input_beat_value.get()) / divisor))
    seconds_per_beat = 60 / float(input_bpm.get())
    hertz = seconds_per_beat * (float(input_beat_value.get()) / divisor);
    
    line1 = "%s. % 6i ms % 6.2f Hz\n" % (divisor_string, (milliseconds * 1.5),
                                          (1 / (hertz * 1.5)))
    line2 = "%s  % 6i ms % 6.2f Hz\n" % (divisor_string, (milliseconds * 1.0),
                                          (1 / (hertz * 1.0)))
    line3 = "%st % 6i ms % 6.2f Hz" % (divisor_string, (milliseconds * .67),
                                        (1 / (hertz * .67)))
    string = line1 + line2 + line3
    return string

# Note to Frequency functions
def note2freq(arg):
    # Calcualte frequency from a note value
    note_name = ""
    octave = slide_octave.get()
    
    if slide_note.get() == 1: x = 3; note_name = "C"
    elif slide_note.get() == 2: x = 4; note_name = "C#"
    elif slide_note.get() == 3: x = 5; note_name = "D"
    elif slide_note.get() == 4: x = 6; note_name = "D#"
    elif slide_note.get() == 5: x = 7; note_name = "E"
    elif slide_note.get() == 6: x = 8; note_name = "F"
    elif slide_note.get() == 7: x = 9; note_name = "F#"
    elif slide_note.get() == 8: x = 10; note_name = "G"
    elif slide_note.get() == 9: x = 11; note_name = "G#"
    elif slide_note.get() == 10: x = 0; note_name = "A"; octave = octave + 1
    elif slide_note.get() == 11: x = 1; note_name = "A#"; octave = octave + 1
    elif slide_note.get() == 12: x = 2; note_name = "B"; octave = octave + 1

    y = slide_concert.get() / float(2 ** (5 - octave))
    frequency = 2 ** (x / float(12)) * y
    # The following alternate formula is taken from:
    # http://en.wikipedia.org/wiki/
    # Cent_%28music%29#Piecewise_linear_approximation
    # c = frequency * (1 + .0005946 * cents)
    frequency_with_cents = frequency * 2 ** (slide_cent.get() / float(1200))

    # Show selected note value, frequency
    concert_label = str(slide_concert.get())
    octave_label = str(slide_octave.get())
    cent_label = str(slide_cent.get())
    
    label_note = note_name + octave_label + "." + cent_label
    note_value_display.set(label_note)
    freq_value_display.set(round(frequency_with_cents, 2))
    gen_value_display.set(round(frequency_with_cents, 2))
    concert_value_display.set("Concert A (" + concert_label + ")")

# Frequency generator functions
def gen_play():
    # Get the freqency from user input or n2f converter,
    # then play it for a specified time in seconds.
    validate()
    gen_sound = tkSnack.Sound(rate=44100)
    gen_freq = float(input_gen_freq.get())
    gen_vol = gen_vol_slider.get() * 3275
    gen_length = int(float(input_gen_length.get()) * 44100)
    tone = tkSnack.Filter('generator', gen_freq, gen_vol,
                            0.0, 'sine', gen_length)
    
    if gen_button["text"] == "Play":
        gen_button["text"] = "Stop"
        # Stop the metronome
        metro_button["text"] = "Start"
        global play
        play = False
        # Play the frequency
        gen_sound.play(filter=tone)
        root.after(int(float(input_gen_length.get())) * 1000, gen_stop)
    else:
        gen_stop()
        tone.destroy()

def gen_stop():
    # Stop the sound and reset the button
    gen_button["text"] = "Play"
    tkSnack.audio.stop()

def gen_vol_adjust(arg):
    # This is a hacked way to make the volume slider work while
    # the sound is playing. For some reason tkSnack.audio.play_gain()
    # was not working for me. Basically, restarting the sound with
    # a different volume.
    if gen_button["text"] == "Stop":
        gen_sound = tkSnack.Sound(rate=44100)
        gen_freq = float(input_gen_freq.get())
        gen_vol = gen_vol_slider.get() * 3275
        gen_length = int(float(input_gen_length.get()) * 44100)
        tone = tkSnack.Filter('generator', gen_freq, gen_vol,
                                0.0, 'sine', gen_length)
        tkSnack.audio.stop()
        gen_sound.play(filter=tone)

# GUI functions
def validate():
    # Check to see if all fields are valid.
    try:
        if float(input_bpm.get()) <= 0:
            input_bpm.delete(0, END)
            input_bpm.insert(0, 1)
    except:
        input_bpm.delete(0, END)
        input_bpm.insert(0, 120)
    try:
        if int(input_beats_per_measure.get()) <= 0:
            input_beats_per_measure.delete(0, END)
            input_beats_per_measure.insert(0, 1)
    except:
        input_beats_per_measure.delete(0, END)
        input_beats_per_measure.insert(0, 4)
    try:
        x = int(input_beat_value.get())
        if not (x == 1 or x == 2 or x == 4 or
                x == 8 or x == 16 or x ==32 or x == 64):
            warning_note_value()
    except:
        warning_note_value()
    try:
        if float(input_total_measures.get()) < 0:
            input_total_measures.delete(0, END)
            input_total_measures.insert(0, 0)
    except:
        input_total_measures.delete(0, END)
        input_total_measures.insert(0, 0)
    try:
        if ((float(input_gen_freq.get()) > 20000) or
            (float(input_gen_freq.get()) < 20)):
            input_gen_freq.delete(0, END)
            input_gen_freq.insert(0, 440)
    except:
        input_gen_freq.delete(0, END)
        input_gen_freq.insert(0, 440)
    try:
        if (float(input_gen_length.get()) > 4444):
            input_gen_length.delete(0, END)
            input_gen_length.insert(0, 4444)
        elif (float(input_gen_length.get()) < 1):
            input_gen_length.delete(0, END)
            input_gen_length.insert(0, 1)
    except:
        input_gen_length.delete(0, END)
        input_gen_length.insert(0, 1)

def warning_note_value():
    # Give the user a clue about note resolutions.
    tkMessageBox.showwarning("Note Value", "Use the following:\n\n" +
    " (1)  Whole note\n (2)  Half note\n (4)  Quarter note\n" +
    " (8)  Eighth note\n(16) Sixteenth note\n" +
    "(32) Thirty-second note\n(64) Sixty-fourth note", icon="error")
    input_beat_value.delete(0, END)
    input_beat_value.insert(0, "4")
    input_beat_value.focus_set()

def about():
    # About this program.
    about_string = (
    "RASP Aids Song Production\n" +
    "Version 2.1\n\n" +
    "Copyright 2012\n" +
    "Brian Hilmers\n" +
    "License: OSL 3.0"
    )
    top = Toplevel()
    top.grab_set()
    top.geometry()
    top.resizable(0, 0)
    top.title("RASP - About")
    label = Label(top, text=about_string, padx=20, pady=10)
    label.pack()


# Input Frame: Tap tempo, all inputs, calcualte, song time output
inputframe = Frame(pady=5)
inputframe.pack()

# Column 0: Metronone
metro_frame = LabelFrame(inputframe, text="Metronome", labelanchor=N)
metro_frame.grid(row=0, column=0, rowspan=4, ipadx=2)

tempo_list_default = StringVar(root)
tempo_list_default.set("Animato")
tempo_list = OptionMenu(metro_frame, tempo_list_default, "Grave", "Largo", 
                "Lento", "Adagio", "Larghetto", "Adagietto", "Andante",
                "Andantino", "Maestoso", "Moderato", "Allegretto", "Animato", 
                "Allegro", "Assai", "Vivace", "Presto", "Prestissimo", 
                command=tempo_markings)
tempo_list.grid(row=0, column=0, columnspan=1, sticky=EW)

downbeat = IntVar()
downbeat_button = Checkbutton(metro_frame, text="Downbeat",
                                variable=downbeat, indicatoron=0)
downbeat_button.grid(row=1, column=0, sticky=EW)

metro_vol_slider = Scale(metro_frame, from_=0, to=10, label="Volume",
                        length=130, width=10, orient=HORIZONTAL, showvalue=0)
metro_vol_slider.grid(row=2, column=0, rowspan=2)
metro_vol_slider.set(5)

metro_button = Button(inputframe, text="Start", width=14, command=metro_start)
metro_button.grid(row=4, column=0, rowspan=2)

# Column 1 & 2: Song details
input_bpm = Entry(inputframe, width=5, justify=CENTER)
input_bpm.insert(0, "120")
input_bpm.focus_set()
input_bpm.grid(row=0, column=1, padx=5)
input_beats_per_measure = Entry(inputframe, width=5, justify=CENTER)
input_beats_per_measure.insert(0, "4")
input_beats_per_measure.grid(row=1, column=1)
input_beat_value = Entry(inputframe, width=5, justify=CENTER)
input_beat_value.insert(0, "4")
input_beat_value.grid(row=2, column=1)
input_total_measures = Entry(inputframe, width=5, justify=CENTER)
input_total_measures.insert(0, "0")
input_total_measures.grid(row=3, column=1)

label_beats_per_minute = Label(inputframe, text="BPM")
label_beats_per_minute.grid(row=0, column=2, sticky=W)
label_beats_per_measure = Label(inputframe, text="Beats", justify=LEFT)
label_beats_per_measure.grid(row=1, column=2, sticky=W)
label_beats_per_measure = Label(inputframe, text="Value", justify=LEFT)
label_beats_per_measure.grid(row=2, column=2, sticky=W)
label_total_measures = Label(inputframe, text="Measures")
label_total_measures.grid(row=3, column=2, sticky=W)

label_delay_selector = Label(inputframe, text="Delay Length")
label_delay_selector.grid(row=4, column=1, columnspan=2)

delay_range = IntVar()
radio_long = Radiobutton(inputframe, indicatoron=0,
                          width=5, text="long", variable=delay_range, value=0)
radio_long.grid(row=5, column=1, sticky=E)
radio_short = Radiobutton(inputframe, indicatoron=0,
                        width=5, text="short", variable=delay_range, value=1)
radio_short.grid(row=5, column=2, sticky=W)
radio_short.select()

# Column 3: Song time display, calculate button
label_song_time_frame = LabelFrame(inputframe, text="Song Time", labelanchor=N)
label_song_time_frame.grid(row=0, column=3, rowspan=4, padx=5)
song_time_output = StringVar()
song_time_output.set("0:00")
large_font = tkFont.Font(size=16, weight="bold")
label_song_time = Label(label_song_time_frame, textvariable=song_time_output,
                      font=large_font, width=9, height=3).pack()

button_calculate = Button(inputframe, text="Calculate",
                          command=calculate, width=14)
button_calculate.grid(row=4, column=3, rowspan=2)

# Delay Frame: Delay calcuation output --> populate()
delayframe = Frame()
delayframe.pack()

# Column 0
label_1_frame = LabelFrame(delayframe)
label_1_frame.grid(row=1, column=0)
dcalc_1 = StringVar()
dcalc_1.set("\n\n")
label_1 = Label(label_1_frame, textvariable=dcalc_1, justify=RIGHT,
                width=28, font="TkFixedFont").pack()
label_2_frame = LabelFrame(delayframe)
label_2_frame.grid(row=2, column=0)
dcalc_2 = StringVar()
dcalc_2.set("\n\n")
label_2 = Label(label_2_frame, textvariable=dcalc_2, justify=RIGHT,
                width=28, font="TkFixedFont").pack()
label_3_frame = LabelFrame(delayframe)
label_3_frame.grid(row=3, column=0)
dcalc_3 = StringVar()
dcalc_3.set("\n\n")
label_3 = Label(label_3_frame, textvariable=dcalc_3, justify=RIGHT,
                width=28, font="TkFixedFont").pack()

# Column 1
label_4_frame = LabelFrame(delayframe)
label_4_frame.grid(row=1, column=1)
dcalc_4 = StringVar()
dcalc_4.set("\n\n")
label_4 = Label(label_4_frame, textvariable=dcalc_4, justify=RIGHT,
                width=28, font="TkFixedFont").pack()
label_5_frame = LabelFrame(delayframe)
label_5_frame.grid(row=2, column=1)
dcalc_5 = StringVar()
dcalc_5.set("\n\n")
label_5 = Label(label_5_frame, textvariable=dcalc_5, justify=RIGHT,
                width=28, font="TkFixedFont").pack()
label_6_frame = LabelFrame(delayframe)
label_6_frame.grid(row=3, column=1)
dcalc_6 = StringVar()
dcalc_6.set("\n\n")
label_6 = Label(label_6_frame, textvariable=dcalc_6, justify=RIGHT,
                width=28, font="TkFixedFont").pack()

# Note-to-Frequecny Frame: Independent of calulate function --> note2freq()
n2f_frame = Frame()
n2f_frame.pack()

slide_concert = Scale(n2f_frame, from_=435, to=445, label="Concert A",
                        length=240, width=10, orient=HORIZONTAL,
                        showvalue=0, command=note2freq)
slide_concert.grid(row=0, column=0)
slide_note = Scale(n2f_frame, from_=1, to=12, label="Note",
                        length=240, width=10,  orient=HORIZONTAL,
                        showvalue=0, command=note2freq)
slide_note.grid(row=1, column=0)
slide_octave = Scale(n2f_frame, from_=0, to=8, label="Octave",
                        length=240, width=10,  orient=HORIZONTAL,
                        showvalue=0, command=note2freq)
slide_octave.grid(row=2, column=0)
slide_cent = Scale(n2f_frame, from_=-50, to=50, label="Cents",
                        length=240, width=10,  orient=HORIZONTAL,
                        showvalue=0, command=note2freq)
slide_cent.grid(row=3, column=0)

# Set default values to A4.0 @ 440Hz
slide_concert.set(440)
slide_note.set(10)
slide_octave.set(4)
slide_cent.set(0)

note_value_display = StringVar()
concert_value_display = StringVar()
concert_value_display.set(slide_concert.get())
note_value_frame = LabelFrame(n2f_frame, text="Note", labelanchor=N)
note_value_frame.grid(row=0, column=1, rowspan=2, padx=2)
note_value_display_label = Label(note_value_frame, font=large_font, width=10,
                                 height=1, textvariable=note_value_display)
note_value_display_label.pack()
concert_value_display_label = Label(note_value_frame,
                                    textvariable=concert_value_display)
concert_value_display_label.pack()

freq_value_display = StringVar()
freq_value_frame = LabelFrame(n2f_frame, text="Frequency (Hz)", labelanchor=N)
freq_value_frame.grid(row=2, column=1, rowspan=2, padx=2)
freq_value_display_label = Label(freq_value_frame, font=large_font, width=10,
                                 height=2, textvariable=freq_value_display)
freq_value_display_label.pack()


# Frequecny Generator Frame: Takes one value from note2freq.
gen_frame = LabelFrame(root, text="Frequency Generator", labelanchor=N)
gen_frame.pack(ipadx=2, ipady=2)

gen_vol_slider = Scale(gen_frame, from_=0, to=10, label="Volume", length=235,
            width=10, orient=HORIZONTAL, showvalue=0, command=gen_vol_adjust)
gen_vol_slider.grid(row=0, column=0, rowspan=2)
gen_vol_slider.set(5)

gen_value_display = StringVar()
input_gen_freq = Entry(gen_frame, width=5, justify=CENTER,
                        textvariable=gen_value_display)

input_gen_freq.grid(row=0, column=1, padx=5)
label_gen_freq = Label(gen_frame, text="Freq")
label_gen_freq.grid(row=0, column=2, sticky=W)

input_gen_length = Entry(gen_frame, width=5, justify=CENTER)
input_gen_length.insert(0, "1")
input_gen_length.grid(row=1, column=1)
label_gen_length = Label(gen_frame, text="Sec", justify=LEFT)
label_gen_length.grid(row=1, column=2, sticky=W)

gen_button = Button(gen_frame, text="Play", width=5, height=2, command=gen_play)
gen_button.grid(row=0, column=3, rowspan=2, padx=2)


# Info Frame: About RASP
bottom_frame = Frame()
bottom_frame.pack(pady=5)
info_frame = LabelFrame(bottom_frame)
info_frame.grid(row=0, column=0)
info_label = Label(info_frame,
        text="Tap tempo with spacebar, arrows adjust BPM by 1 or 10.",width=45)
info_label.pack()
button_about = Button(bottom_frame, text="?", command=about)
button_about.grid(row=0, column=1)


# Launch the GUI, bind spacebar to main window.
root.bind("<space>", tap)
root.bind("<Key>", arrow)
root.title("RASP")
root.geometry("425x575+50+50")
root.resizable(0, 0)
root.mainloop()
